@@ -8,8 +8,8 @@ xgobi.colors.default <-
 
 xgobi <-
 function(matrx,
-	 collab = dimnames(matrx)[[2]],
-	 rowlab = dimnames(matrx)[[1]],
+	 collab = colnames(matrx),
+	 rowlab = rownames(matrx),
 	 colors = NULL,
 	 glyphs = NULL,
 	 erase	= NULL,
@@ -22,6 +22,7 @@ function(matrx,
 	 nlinkable  = NULL,
 	 subset = NULL,
 	 display= NULL,
+         multi  = TRUE,
 	 keep	= FALSE,
 	 fprefix= "xgobi-")
 {
@@ -37,11 +38,15 @@ function(matrx,
     dfile <- tempfile(paste(fprefix,
                             abbreviate(gsub("[^A-Za-z0-9]","",title), 5),
                             sep=""))
+
+    if(.Platform$OS.type == "windows") dfile <- gsub("\\\\", "/", dfile)
+
     write.table(x, file = dfile, quote = FALSE,
 		row.names = FALSE, col.names = FALSE)
-    if(!keep) on.exit(unlink(dfile), add = TRUE)
+    tmpfiles <- dfile
+    if(!keep) on.exit(file.remove(tmpfiles))
 
-    args <- paste("-std", std) ##, "-dev", dev)
+    args <- paste("-std", std)
 
     ## Column / Var labels ###
     if (!is.null(collab)) {
@@ -49,8 +54,8 @@ function(matrx,
 	    stop("The `collab' argument needs to be a character vector")
 	if (!missing(collab) && length(collab) != NCOL(x))
 	    stop("`collab' has wrong length (not matching NCOL(x))")
-        cat(collab, file = (colfile <- paste(dfile, ".col", sep="")), sep="\n")
-        if(!keep) on.exit(unlink(colfile), add = TRUE)
+        cat(collab, file = (colfile <- paste(dfile, ".col", sep="")), sep= "\n")
+        tmpfiles <- c(tmpfiles, colfile)
     }
     ## Row / Case labels ###
     if (!is.null(rowlab)) {
@@ -59,21 +64,21 @@ function(matrx,
 	if (!missing(rowlab) && length(rowlab) != NROW(x))
 	    stop("`rowlab' has wrong length (not matching NROW(x))")
         cat(rowlab, file = (rowfile <- paste(dfile, ".row", sep="")), sep="\n")
-        if(!keep) on.exit(unlink(rowfile), add = TRUE)
+        tmpfiles <- c(tmpfiles, rowfile)
     }
     ## Variable groups ##
     if (!is.null(vgroups)) {
-	   if (!is.vector(vgroups) || !is.numeric(vgroups))
-	       stop("The `vgroups' argument needs to be a numeric vector")
-	   cat(vgroups, file=(vgfile <- paste(dfile,".vgroups",sep="")), sep="\n")
-           if(!keep) on.exit(unlink(vgfile), add = TRUE)
+	if (!is.vector(vgroups) || !is.numeric(vgroups))
+	    stop("The `vgroups' argument needs to be a numeric vector")
+	cat(vgroups, file=(vgfile <- paste(dfile,".vgroups",sep="")), sep="\n")
+        tmpfiles <- c(tmpfiles, vgfile)
     }
     ## Colors ##
     if (!is.null(colors)) {
 	if (!is.vector(colors) || !is.character(colors))
 	    stop("The `colors' argument needs to be a character vector")
 	cat(colors, file = (clrfile <- paste(dfile,".colors",sep="")), sep="\n")
-        if(!keep) on.exit(unlink(clrfile), add = TRUE)
+        tmpfiles <- c(tmpfiles, clrfile)
     }
     ## Glyphs ##
     if (!is.null(glyphs)) {
@@ -81,7 +86,7 @@ function(matrx,
 	    stop("The `glyphs' argument needs to be a numeric vector")
 	glyphfile <- paste(dfile, ".glyphs", sep = "")
 	cat(glyphs, file = glyphfile, sep = "\n")
-        if(!keep) on.exit(unlink(glyphfile), add = TRUE)
+        tmpfiles <- c(tmpfiles, glyphfile)
     }
     ## Erase ##
     if (!is.null(erase)) {
@@ -89,7 +94,7 @@ function(matrx,
 	    stop("The `erase' argument needs to be a numeric vector")
 	erasefile <- paste(dfile, ".erase", sep = "")
 	cat(erase, file = erasefile, sep = "\n")
-        if(!keep) on.exit(unlink(erasefile), add = TRUE)
+        tmpfiles <- c(tmpfiles, erasefile)
     }
     ## Connected lines ##
     if (!is.null(lines)) {
@@ -101,7 +106,7 @@ function(matrx,
 	    for (i in 1:nrow(lines))
 		cat(lines[i, ], "\n", file = linesfile, append = TRUE)
 	}
-        if(!keep) on.exit(unlink(linesfile), add = TRUE)
+        tmpfiles <- c(tmpfiles, linesfile)
 
 	## Line colors ##
 	if (!is.null(linecolors)) {
@@ -109,8 +114,7 @@ function(matrx,
 		stop("The `linecolors' argument must be a character vector")
 	    linecolorfile <- paste(dfile, ".linecolors", sep = "")
 	    cat(linecolors, file = linecolorfile, sep = "\n")
-
-            if(!keep) on.exit(unlink(linecolorfile), add = TRUE)
+            tmpfiles <- c(tmpfiles, linecolorfile)
 	}
     }
     ## Resources ##
@@ -119,7 +123,7 @@ function(matrx,
 	    stop("The `resources' argument must be a character vector")
 	resourcefile <- paste(dfile, ".resources", sep = "")
 	cat(resources, file = resourcefile, sep = "\n")
-        if(!keep) on.exit(unlink(resourcefile), add = TRUE)
+        tmpfiles <- c(tmpfiles, resourcefile)
     }
     ## nlinkable ##
     if (!is.null(nlinkable)) {
@@ -128,7 +132,7 @@ function(matrx,
 	    stop("The `nlinkable' argument must be a scalar integer")
 	linkablefile <- paste(dfile, ".nlinkable", sep = "")
 	cat(nlinkable, "\n", file = linkablefile)
-        if(!keep) on.exit(unlink(linkablefile), add = TRUE)
+        tmpfiles <- c(tmpfiles, linkablefile)
     }
     ## subset ##
     subsetarg <- ""
@@ -147,18 +151,28 @@ function(matrx,
 	    warning("display must be a character string")
 	else args <- paste("-display", display, args)
     }
-    args <- paste("-title", paste("'", title, "'", sep = ""), args)
 
+    if(.Platform$OS.type == "windows") {
+        args <- paste("-vtitle", paste("'", title, "'", sep = ""), args)
+        xgpath <- system.file("scripts", "xgobi.bat", package="xgobi")
+        command <- paste(xgpath, args, dfile)
+        cat(command, "\n")
+
+        s <- if(multi) system(command, wait=FALSE, minimized=TRUE)
+        else system(command, intern=FALSE, wait=TRUE, minimized=TRUE)
+    } else {
+        args <- paste("-title", paste("'", title, "'", sep = ""), args)
 ### Note to installer:
 ### Here you will need to specify the path to the xgobi executable
 ### on your system (here we assume it *is* in the user's PATH :)
 
-    command <- paste("xgobi", args, dfile, "&")
+        xgpath <- "xgobi"
+        command <- paste(xgpath, args, dfile, if(multi) "&")
+        cat(command, "\n")
 
-    cat(command, "\n")
-    s <- system(command, FALSE)
-
-    ## Now wait a bit before unlinking all the files via  on.exit(.) :
-    if(!keep) Sys.sleep(3)
+        s <- system(command, FALSE)
+    }
+    ## Now wait a bit to allow xgobi to start and read its files
+    if(multi && !keep) Sys.sleep(3)
     invisible(s)
 }
